/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include <wchar.h>
#include "acceptbaseparser.h"
#include "misc.h"



namespace Winix
{




bool AcceptBaseParser::IsWhite(int c)
{
	if( c==' ' || c=='\t' )
		return true;

return false;
}


void AcceptBaseParser::SkipWhite()
{
	while( IsWhite(*text) )
		++text;
}




void AcceptBaseParser::ReadParameter()
{
	param.clear();
	SkipWhite();

	while( *text!=0 && *text!=',' && *text!=';' )
	{
		param += *text;
		++text;
	}

	TrimWhite(param);
}


void AcceptBaseParser::ReadQ()
{
	q = 1.0;
	SkipWhite();

	if( *text != ';' )
		return;

	++text; // skipping a semicolon

	while( *text!=0 && *text!=',' && *text!='=' )
		// skipping until ',' or '='
		++text;

	if( *text==0 || *text==',' )
		return;

	++text; // skipping '='

	SkipWhite();
	q = wcstod(text, (wchar_t**)&text);
}


void AcceptBaseParser::SkipParam()
{
	SkipWhite();

	if( *text == ',' )
		++text;
}



void AcceptBaseParser::Parse(const wchar_t * str)
{
	text = str;
	Init();
	
	while( *text != 0 )
	{
		ReadParameter();
		ReadQ();
		SkipParam();
		Param(param, q);
	}
}



void AcceptBaseParser::Parse(const std::wstring & str)
{
	Parse(str.c_str());
}



} // namespace Winix

