/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2018, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "groups.h"


namespace Winix
{



Groups::Groups()
{
	Clear();
}


void Groups::set_dependency(WinixModel * winix_model)
{
	WinixModel::set_dependency(winix_model);
	table.set_dependency(winix_model);
}


void Groups::Clear()
{
	table.Clear();
}


void Groups::ReadGroups(Db * db)
{
	Clear();
	
	db->GetGroups(table);	
}


Group * Groups::GetGroup(long group_id)
{
	Table::Iterator i = table.FindId(group_id);
	
	if( i == table.End() )
		return 0;
	
return &(*i);
}


Group * Groups::GetGroup(const std::wstring & name)
{
	Table::Iterator i = table.FindName(name);
	
	if( i == table.End() )
		return 0;
	
return &(*i);
}


long Groups::GetGroupId(const std::wstring & name)
{
	Group * pgroup = GetGroup(name);
	
	if( !pgroup )
		return -1;	
	
return pgroup->id;
}



Groups::Iterator Groups::Begin()
{
	return table.Begin();
}



Groups::Iterator Groups::End()
{
	return table.End();
}



Groups::SizeType Groups::Size()
{
	return table.Size();
}


bool Groups::Remove(long group_id)
{
	return table.Remove(group_id);
}




} // namespace Winix

