/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2012-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_core_jobs
#define headerfile_winix_core_jobs

#include <vector>
#include <queue>
#include "basethread.h"
#include "space/space.h"


namespace Winix
{



#define WINIX_JOBS_HOW_MANY_PRIORITIES	32




class Job : public BaseThread
{
public:

	Job();

	/*
		add a new job to the queue
		priority: 0-31 (0 - the lowest priority, 31 - the highest priority)
	*/
	void Add(PT::Space & job, int priority = 0);


	/*
		queue size, and size of all jobs in any priority
	*/
	size_t Size(int priority) const;
	size_t Size() const;


	/*
		true if specified queue is empty
		or if all queues are empty
	*/
	bool Empty(int priority) const;
	bool Empty() const;


private:

	typedef std::queue<PT::Space>  JobsQueue;
	typedef std::vector<JobsQueue> JobsQueueTab;
	JobsQueueTab jobs_queue_tab;

	void CheckPriority(int & priority) const;
	void SaveToFile();
	void ReadFromFile();


	/*
		second thread
	*/

	// standard winix jobs
	// Image image;
	// sending emails
	// etc.

	bool SignalReceived();
	void Do();
	void DoQueue(JobsQueue & jobs_queue);
	void DoJob(PT::Space & job);
	void DoWinixJob(PT::Space & job);

};


} // namespace Winix


#endif

