/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2018, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_core_postparser
#define headerfile_winix_core_postparser

#include <fcgiapp.h>
#include <string>
#include "httpsimpleparser.h"
#include "requesttypes.h"
#include "misc.h"
#include "utf8/utf8.h"
#include "convert/text.h"


namespace Winix
{



class PostParser : public HttpSimpleParser
{
	FCGX_Stream * in;
	PostTab * post_tab;
	size_t log_value_size;
	int var_index;
	bool has_winix_post_params_msg;
	bool has_winix_raw_post_msg;
	std::string raw_post;

protected:


	virtual int GetChar()
	{
		int c = FCGX_GetChar(in);

		if( c != -1 && has_winix_raw_post_msg )
			raw_post += c;

	return c;
	}
	

	void CreateLog(bool param_added, const std::wstring & name, const std::wstring & value)
	{
		log << log2 << "Method POST, name: \"" << name << "\"";
		
		if( log_value_size > 0 && !PT::IsSubStringNoCase(L"pass", name.c_str()) )
		{
			log << ", value: ";
			
			if( value.size() > log_value_size )
				log << "(first " << log_value_size << " characters) ";
			
			log << "\"";
			log.LogString(value, log_value_size);
			log << "\" (size: " << value.size() << ")";
		}

		if( param_added == false )
			log << log2 << " (skipped)";
			
		log << log2 << logend;
	}



	virtual void Parameter(std::wstring & name, std::wstring & value)
	{
	bool added;
	std::pair<PostTab::iterator, bool> res;

		if( has_winix_post_params_msg )
			plugin->Call(0, WINIX_POST_PARAMS, &name, &value);

		res = post_tab->insert( std::make_pair(name, value) );
		added = res.second;

		if( !added )
		{
			name += L"_inc";
			name += Toa(var_index);
			res = post_tab->insert( std::make_pair(name, value) );
			added = res.second;
			var_index += 1;
		}

		CreateLog(added, name, value);
	}
	

public:

	PostParser()
	{
		log_value_size = 0;
		HttpSimpleParser::getchar_returns_utf8_chars = true;
	}

	void LogValueSize(size_t s)
	{
		log_value_size = s;
	}

	void Parse(FCGX_Stream * in_, PostTab & post_tab_)
	{
		in = in_;
		post_tab = &post_tab_;
		var_index = 1;
		raw_post.clear();

		has_winix_post_params_msg = plugin->HasMessage(WINIX_POST_PARAMS);
		has_winix_raw_post_msg    = plugin->HasMessage(WINIX_RAW_POST_STRING);

		HttpSimpleParser::Parse();

		if( has_winix_raw_post_msg )
			plugin->Call(0, WINIX_RAW_POST_STRING, &raw_post);

		raw_post.clear();
	}

};



} // namespace Winix


#endif
