/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_db_db
#define headerfile_winix_db_db

#include <vector>
#include <map>
#include <cstdio>
#include <ctime>
#include <cstring>
#include "dbbase.h"
#include "core/user.h"
#include "core/group.h"
#include "core/dircontainer.h"
#include "core/ugcontainer.h"


namespace Winix
{


class Db : public DbBase
{
public:

	Db()
	{
		is_postgresql_smaller_than_10 = false;
	}

	void PostgreSQLsmallerThan10(bool is_smaller_than_10);

	bool  GetUserPass(const std::wstring & login, long & user_id, UserPass & up);
	Error AddUser(User & user, const UserPass & up);
	Error ChangeUserPass(long user_id, const UserPass & up);
	Error ChangeUserEnv(long user_id, const PT::Space & space);
	Error ChangeUserAdminEnv(long user_id, const PT::Space & space);
	Error ChangeUserStatus(long user_id, int status);
	Error ChangeUserEmail(long user_id, const std::wstring & email);
	Error ChangeUserLocale(long user_id, size_t locale_id);
	Error ChangeUserTimeZone(long user_id, size_t time_zone_id);
	Error RemoveUser(long user_id);

	void GetUsers(UGContainer<User> & user_tab);
	void GetGroups(UGContainer<Group> & group_tab);


protected:

	DbTextStream query, query_create_url;
	bool is_postgresql_smaller_than_10;
	std::wstring postgrsql_row_statement;

};



} // namespace Winix


#endif

