/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2010-2018, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_db_dbbase
#define headerfile_winix_db_dbbase


#include "dbconn.h"
#include "dbtextstream.h"
#include <vector>
#include <string>
#include "core/error.h"
#include "space/spaceparser.h"
#include "date/date.h"
#include "core/winixbase.h"



namespace Winix
{



class DbBase : public WinixBase
{
public:

	DbBase();

	void SetConn(DbConn * conn);
	void SetConn(DbConn & conn);
	DbConn * GetConn();

	void LogQueries(bool log_q);

	PGresult * AssertQuery(const char * query);
	PGresult * AssertQuery(const DbTextStream & query);
	PGresult * AssertQuery(const char * q, ExecStatusType t);
	PGresult * AssertQuery(const DbTextStream & query, ExecStatusType t);
	void AssertResult(PGresult * r, ExecStatusType t);
	int  AssertColumn(PGresult * r, const char * column_name);

	const char *  AssertValue(PGresult * r, int row, int col);
	const std::wstring & AssertValueWide(PGresult * r, int row, int col); // warning: this method uses a static buffer
	void          AssertValueWide(PGresult * r, int row, int col, std::wstring & result);
	void          AssertValueBin(PGresult * r, int row, int col, std::string & result);
	long          AssertValueLong(PGresult * r, int row, int col);
	int           AssertValueInt(PGresult * r, int row, int col);
	bool          AssertValueBool(PGresult * r, int row, int col);
	unsigned long AssertValueULong(PGresult * r, int row, int col);
	unsigned int  AssertValueUInt(PGresult * r, int row, int col);
	PT::Date      AssertValueDate(PGresult * r, int row, int col);

	bool AssertValueSpace(PGresult * r, int row, int col, PT::Space & space);

	void ClearResult(PGresult * r);
	long AssertCurrval(const char * table);

	bool IsNull(PGresult * r, int row, int col);
	int Rows(PGresult * r);
	int Cols(PGresult * r);
	long AffectedRows(PGresult * r);

	void CreateIdList(const std::vector<long> & id_tab, std::wstring & list, bool add_parentheses = true);

	Error DoCommand(const DbTextStream & command);
	Error DoCommand(const char * command);
	Error BeginTrans();
	Error RollbackTrans();
	Error CommitTrans();
	Error EndTrans(Error err);

	// new API (returns only bool)
	bool EndTrans(bool everything_ok);

	void UnescapeBin(const char * str, size_t len, std::string & out, bool clear_out = true);


protected:

	// a helper method for escaping strings
	template<class RawType>
	DbTextStream::RawText<RawType> R(const RawType & par)
	{
		return DbTextStream::RawText<RawType>(par);
	}

	DbConn * db_conn;
	bool log_queries;


private:

//	static int  CharToInt(char c);
//	static bool IsCorrectOctalDigit(char c);
//	static int  UnescapeBin(const char * str, size_t & i, size_t len);

	char UnescapeBinHexToDigit(char hex);

	DbTextStream bquery;
	PT::SpaceParser conf_parser;

};



} // namespace Winix



#endif

