/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "emacs.h"
#include "templates/templates.h"
#include "functions.h"



namespace Winix
{


namespace Fun
{

Emacs::Emacs()
{
	fun.url = L"emacs";
}



bool Emacs::HasAccess(const Item & item)
{
	if( cur->session->puser && cur->session->puser->super_user )
		// super user can use emacs everywhere
		return true;

	if( !system->HasWriteAccess(item) )
		return false;

	if( !system->mounts.pmount->IsPar(system->mounts.MountParEmacsOn()) )
		return true;

	if( system->mounts.pmount->IsArg(system->mounts.MountParEmacsOn(), cur->request->dir_tab.size()) )
		return true;	
	
return false;
}



bool Emacs::HasAccess()
{
	return HasAccess(*cur->request->last_item);
}





// !! IMPROVE ME in functions.cpp there is a similar function
/*
bool Emacs::PostEmacsCheckAbuse(bool adding)
{
	if( !system->rebus.CheckRebus() )
	{
		cur->request->status = WINIX_ERR_INCORRECT_REBUS;
		return false;
	}

	// !! is tested in createthread once
	functions->CheckGetPostTimes();

	if( cur->session->spam_score > 0 )
	{
		cur->request->status = WINIX_ERR_SPAM;
		log << log1 << "Emacs: ignoring due to suspected spamming" << logend;
		return false;
	}

return true;
}
*/


int Emacs::NotifyCodeEdit()
{
// !! nie potrzebne
//	if( system->mounts.pmount->type == system->mounts.MountTypeThread() )
//		return WINIX_NOTIFY_CODE_THREAD_POST_CHANGED;

return WINIX_NOTIFY_CODE_EDIT;
}




int Emacs::NotifyCodeAdd()
{
// !! nie potrzebne
//	if( system->mounts.pmount->type == system->mounts.MountTypeThread() )
//		return WINIX_NOTIFY_CODE_THREAD_REPLAYED;

return WINIX_NOTIFY_CODE_ADD;
}




void Emacs::MakePost()
{
	bool adding = !cur->request->is_item;

	if( !adding )
		old_url = cur->request->item.url;

	functions->ReadItem(cur->request->item, Item::file);

	if( adding )
		functions->SetUser(cur->request->item); // set user before checking the rebus

	if( functions->CheckAbuse() )
		return;

	if( adding )
	{
		cur->request->is_item          = true;	// !! moze lepiej nie ustawiac is_item? (bo jak wystapi blad np dodania do bazy danych
												//    to formularz edycji zmieni sie z 'dodaj' na 'edytuj'
		cur->request->item.item_content.privileges  = system->NewFilePrivileges();
		cur->request->status = system->AddFile(cur->request->item, NotifyCodeAdd());
	}
	else
	{
		if( system->EditFile(cur->request->item, cur->request->item.url != old_url, NotifyCodeEdit()) )
			cur->request->status = WINIX_ERR_OK;
		else
			cur->request->status = WINIX_ERR_PERMISSION_DENIED;
	}

	if( cur->request->status == WINIX_ERR_OK )
	{
		if( adding )
		{
			system->RedirectToLastFunction(nullptr, false);
		}

		functions->CheckSpecialFile(cur->request->item);
	}
	else
	{
		log << log1 << "Emacs: error: "	<< cur->request->status << logend;
	}
}








} // namespace

} // namespace Winix

