/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2010-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_functions_functionbase
#define headerfile_winix_functions_functionbase

#include <string>
#include <vector>
#include "db/db.h"
#include "core/request.h"
#include "core/config.h"
#include "core/system.h"
#include "core/synchro.h"
#include "notify/notify.h"
#include "core/winixrequest.h"
#include "models/item.h"



namespace Winix
{


class Functions;
class Templates;



class FunctionBase : public WinixRequest
{
public:

	FunctionBase();
	virtual ~FunctionBase();


	// user, group, permissions, url (function name)
	Item fun;

	// auto follow sym links, default: true
	bool follow_symlinks;

	// html template index (for using with 'patterns' object)
	size_t template_index;

	// try to use SSL
	// if in the config 'use_ssl' is true and 'use_ssl_only_for_logged_users' is true
	// then ssl is used only for logged users but sometimes there is a need to use
	// SSL even if noone is logged (for example for such functions like 'login' or 'adduser')
	// default: false
	// (this option is ignored if 'use_ssl' in the config is false)
	bool need_ssl;

	// true if the function need a session object
	// if false then a temporary session is used
	bool need_session;

	virtual void Init();
	virtual bool HasAccess();
	virtual void MakePost();
	virtual void MakeGet();
	virtual void MakeDelete();

	//void SetConfig(Config * pconfig);
	//void SetCur(Cur * pcur);
	void SetDb(Db * pdb);
	//void SetSystem(System * psystem);
	void SetFunctions(Functions * pfunctions);
	void SetTemplates(Templates * ptemplates);
	//void SetSynchro(Synchro * psynchro);
	//void SetSessionManager(SessionManager * pmanager);

protected:

	//Config    * config;
	//Cur       * cur;
	Db        * db; // depracated
	//System    * system;
	Functions * functions;
	Templates * templates;
	//Synchro   * synchro;
	//SessionManager * session_manager;

};


} // namespace Winix

#endif
