/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2011-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "meta.h"
#include "core/log.h"


namespace Winix
{


namespace Fun
{

Meta::Meta()
{
	fun.url = L"meta";
	// !! CHECKME what about follow symlinks?
}


bool Meta::HasAccess()
{
	if( cur->request->IsParam(L"a") )
		return cur->session->puser && cur->session->puser->super_user;
	else
		return system->HasWriteAccess(*cur->request->last_item);
}



bool Meta::Parse(const std::wstring & meta_str)
{
	space.clear();
	conf_parser.SetSpace(space);

return (conf_parser.ParseSpace(meta_str) == PT::SpaceParser::ok);
}




bool Meta::EditAdminMeta(Item & item, const std::wstring & meta_str, bool use_ses_log)
{
	if( Parse(meta_str) )
	{
		item.propagate_connector();
		item.item_content.meta_admin = space;

		//if( db->EditAdminMetaById(space, item_id) == WINIX_ERR_OK )
		if( item.item_content.update() )
		{
			return true;
		}
		else
		{
			log << log1 << "Meta: a database problem with changing admin meta information for item id: "
				<< item.id << logend;
		}
	}
	else
	{
		log << log2 << "Meta: Syntax error in line: " << conf_parser.get_last_parsed_line() << logend;

		if( use_ses_log )
			slog << logerror << T("syntax_error_in_line") << ' ' << conf_parser.get_last_parsed_line() << logend;
	}

return false;
}



bool Meta::EditMeta(Item & item, const std::wstring & meta_str, bool use_ses_log)
{
	if( Parse(meta_str) )
	{
		item.propagate_connector();
		item.item_content.meta = space;

		//if( db->EditMetaById(space, item_id) == WINIX_ERR_OK )
		if( item.item_content.update() )
		{
			return true;
		}
		else
		{
			log << log1 << "Meta: a database problem with changing meta information for item id: "
				<< item.id << logend;
		}
	}
	else
	{
		log << log2 << "Meta: Syntax error in line: " << conf_parser.get_last_parsed_line() << logend;

		if( use_ses_log )
			slog << logerror << T("syntax_error_in_line") << ' ' << conf_parser.get_last_parsed_line() << logend;
	}

return false;
}



void Meta::ChangeAdminMeta()
{
	// IMPROVE ME we need to show an error msg if the user is not an admin

	if( cur->session->puser && cur->session->puser->super_user )
	{
		const std::wstring & meta_str = cur->request->PostVar(L"itemmeta");

		if( EditAdminMeta(*cur->request->last_item, meta_str, true) )
		{
			system->RedirectToLastItem();
		}
	}
}


void Meta::ChangeMeta()
{
	if( system->HasWriteAccess(*cur->request->last_item) )
	{
		const std::wstring & meta_str = cur->request->PostVar(L"itemmeta");

		if( EditMeta(*cur->request->last_item, meta_str, true) )
		{
			system->RedirectToLastItem();
		}
	}
}



void Meta::MakePost()
{
	if( cur->request->IsParam(L"a") )
		ChangeAdminMeta();
	else
		ChangeMeta();
}



} // namespace


} // namespace Winix

