/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2011-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_functions_sort
#define headerfile_winix_functions_sort

#include "functionbase.h"

namespace Winix
{


namespace Fun
{


class Sort : public FunctionBase
{
public:

	Sort();
	bool HasAccess();
	void MakeGet();
	void MakePost();


private:
	
	//DbItemQuery iq;
	std::vector<size_t> item_tab; // indexes to cur->request->item_tab

	struct SortPair
	{
		long id;
		int  sort_index;
	};

	std::vector<SortPair> item_tab2; // from post table

	// for sorting item_tab
	struct SortHelper
	{
		Sort * psort;

		SortHelper(Sort * s) : psort(s) {}
		bool operator()(size_t t1, size_t t2);
	};

	// for sorting item_tab2
	static bool SortFun2(const SortPair & s1, const SortPair & s2);

	void GetDirContent();
	void CreateItemTab();
	void CreateItemTab2();
	void UpdateSortIndex(Item & item, int sort_index);
	void UpdateSortIndexes();
};


} // namespace

} // namespace Winix

#endif
