/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2011-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef headerfile_winix_plugins_export_edb
#define headerfile_winix_plugins_export_edb

#include <vector>
#include "db/dbbase.h"
#include "export.h"
#include "core/dirs.h"

namespace Winix
{


namespace Export
{



class EDb : public DbBase
{
public:

	void SetDirs(Dirs * pdirs);

	bool GetExport(long user_id, std::vector<Export> & export_tab, bool clear_tab = true);
	bool GetExport(long id, Export & exp);
	bool GetExportDirs(std::vector<ExportDir> & export_tab, bool clear_tab = true);

private:

	Dirs * dirs;

	Export exp;
	ExportDir exp_dir;
	DbTextStream query;
	std::wstring dir_temp;

	int cid;
	int cuser_id;
	int cdir;
	int cftp_id;
	int cftp_dir;
	int cname;
	int cserver;
	int clogin;
	int cpass;
	int cpass_type;
	int ccan_change_ftp_params;
	int ccan_change_dir;
	int chttp_server;

	void SetExportCols(PGresult * r);
	void SetExportValues(PGresult * r, int row, Export & exp);

};


}

} // namespace Winix

#endif
