/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2010-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_plugins_thread_tdb
#define headerfile_winix_plugins_thread_tdb

#include <vector>
#include "thread.h"
#include "db/dbbase.h"
#include "core/error.h"

namespace Winix
{


namespace Thread
{


class TDb : public DbBase
{
public:

	/*
		high level interface
	*/

	Error AddThread(const Thread & thread);
	Error GetThread(long file_id, Thread & thread);
	Error GetThreads(const std::vector<long> & file_id_tab, std::vector<Thread> & thread_tab);
	Error RemoveThread(long file_id);

	Error AddAnswer(long file_id, long answer_id);
	Error GetAnswers(long file_id, std::vector<long> & answer_id_tab);
	Error RemoveAnswer(long answer_id);


	/*
		low level interface
	*/

	// looking for the last answer in a thread (in 'thread_files' table)
	// this is not the value from 'thread' table
	long FindLastAnswer(long file_id);

	// returning all threads id (file_id) (from 'thread' table)
	void GetAllThreadsId(std::vector<long> & file_id);

	// removing only the answer in 'thread_files' table without updating 'thread' table
	Error RemoveAnswerOnly(long answer_id);

	// returning how many answers there are
	// calculating from 'thread_files' table
	// this is not the value from 'thread' table
	long CalcAnswers(long file_id);

	// recalculating last_item, replies in 'thread' table on a given thread
	Error RecalcThread(long file_id);


private:

	DbTextStream query;
	std::wstring list_id;
	int cfile_id, creplies, cclosed, clast_item, cdate_modification, cuser_id, cguest;
	Thread thread_temp;
	std::vector<long> file_id_tab;

	void  SetThreadColumns(PGresult * r);
	void  SetThread(PGresult * r, int col, Thread & thread);
	Error RemoveAnswerRecalcLast(long file_id);
	Error RemoveAnswerRecalcLast(const std::vector<long> & file_id_tab);

};



} // namespace

} // namespace Winix

#endif

